from datetime import datetime
from enum import Enum
from typing import Tuple, Union
import polars as pl
import sqlalchemy.engine
import sqlparse
from pulseel.utils.datetimes import to_tz_aware
from pulseel.utils.models import ExtractionInfo


class CdcRelationalOperator(str, Enum):
    """Enumeration of the possible relational operators for
    `sys.fn_cdc_map_time_to_lsn` function.

    Possible values:
    - `largest less than`
    - `largest less than or equal`
    - `smallest greater than`
    - `smallest greater than or equal`
    """

    LARGEST_LESS_THAN = "largest less than"
    LARGEST_LESS_THAN_OR_EQUAL = "largest less than or equal"
    SMALLEST_GREATER_THAN = "smallest greater than"
    SMALLEST_GREATER_THAN_OR_EQUAL = "smallest greater than or equal"


def py_hexbin_to_mssql(binary_value: str) -> str:
    """Retorn clausula de conversão a representação hexadecimal para binary do
    SQL Server."""

    return f"CONVERT(BINARY(10),'0x{binary_value[2:] if binary_value.startswith('0x') else binary_value}',1)"


def map_time_to_lsn(
    relational_operator: CdcRelationalOperator,
    timestamp: Union[str, datetime],
    tzinfo: str = "UTC",
) -> str:
    """Retorna a cláusula de conversão de timestamp para LSN do SQL Server.

    Args:
        relational_operator (CdcRelationalOperator): Operador relacional.
        timestamp (Union[str, datetime]): Timestamp a ser convertido.
        tzinfo (str): Timezone para conversão da data. (default: 'UTC').
    """

    tz_aware_timestamp = to_tz_aware(timestamp, tzinfo).replace(tzinfo=None)
    timestamp = tz_aware_timestamp.isoformat(" ", "milliseconds")
    return f"sys.fn_cdc_map_time_to_lsn('{relational_operator}', '{timestamp}')"


def get_schema_from_query(engine: sqlalchemy.engine.Engine, query: str) -> pl.DataFrame:
    return pl.read_database(f"SELECT * FROM ({query}) as schema_query WHERE 1=0", engine)


def normalize_query(engine: sqlalchemy.engine.Engine, query: str, ) -> str:
    schema = get_schema_from_query(engine, query)
    selectables: list[str] = []
    for col, type in zip(schema.columns, schema.dtypes):
        if col.lower().endswith("rowversion"):
            selectables.append(f"CAST({col} AS BIGINT) AS {col}")
        else:
            selectables.append(col)

    return sqlparse.format(f"SELECT {', '.join(selectables)} FROM ({query}) as query")



def get_data(engine: sqlalchemy.engine.Engine,
             query: str) -> Tuple[pl.DataFrame, ExtractionInfo]:

    start = datetime.now()

    query = normalize_query(engine, query)
    df = pl.read_database(query, engine, infer_schema_length=None)

    end = datetime.now()

    exec_time = int((end - start).total_seconds())
    rows, _ = df.shape

    return df, ExtractionInfo(rows_extracted=rows, execution_in_seconds=exec_time)

def get_data_incremental(engine: sqlalchemy.engine.Engine,
                         query: str,
                         inc_column: str,
                         inc_value: str) -> Tuple[pl.DataFrame, ExtractionInfo]:

    start = datetime.now()

    prequery = normalize_query(engine, query)
    final_query = sqlparse.format(f"SELECT * FROM ({prequery}) tpx WHERE {inc_column} > {inc_value}")

    print(final_query)

    data = pl.read_database(final_query, engine, infer_schema_length=None)

    end = datetime.now()


    rows, _ = data.shape
    exec_time = int((end - start).total_seconds())

    if data.is_empty():
        last_value = inc_value
    else:
        last_value = data.select(pl.col(inc_column).max())[0, 0]

    return data, ExtractionInfo(rows_extracted=rows, execution_in_seconds=exec_time, inc_last_value=last_value)
