import logging
import tempfile
import uuid
from dataclasses import dataclass

import adlfs

from pulseel.extractors.mongo import get_mongo_data
from pulseel.loaders.az import upload_to_blob_storage
from pulseel.utils.helpers import get_adls_client
from pulseel.utils.models import MongoConfig, ADLSConfig, ExtractionInfo, IncrementalConfig


def mongo_to_az(mongo_config: MongoConfig,
                az_config: ADLSConfig,
                az_path: str,
                inc_config: IncrementalConfig | None = None):
    """
    Converts data from a MongoDB source to an Azure Data Lake destination.

    This function facilitates the extraction of data from a MongoDB source
    defined by `mongo_config` and moves it to an Azure Data Lake using the
    settings provided in `az_config`. The function then saves the data
    to a specified path in Azure, as determined by `az_path`.

    Parameters:
    mongo_config (MongoConfig): Configuration details and credentials
        for connecting to the MongoDB source.
    az_config (ADLSConfig): Configuration details and credentials for
        connecting to Azure Data Lake Storage (ADLS).
    az_path (str): Destination path in Azure Data Lake where the data
        will be saved.

    Returns:
    ExtractionInfo: Detailed information about the completion of the
        data extraction process, including status and metadata.
    """

    df, info = get_mongo_data(mongo_config.uri, mongo_config.db, mongo_config.collection, incremental=inc_config)

    if info.rows_extracted == 0:
        logging.info("No new data to extracted")
        return


    logging.info(f"Extracted {info.rows_extracted} rows from MongoDB in {info.execution_in_seconds} seconds")

    upload_to_blob_storage(df, az_path, az_config)

    logging.info(
        f"Uploaded {info.rows_extracted} rows to Azure Data Lake at {az_path}"
    )






